import parseurl from 'parseurl';
import finalhandler from 'finalhandler';
import { HttpError } from 'http-errors';
import { getStack, shouldHandle } from '../utils/util.js';
import logger from '../utils/logger.js';
import { createPath, createHeaders } from '../http/http.js';
import * as proxy from '../http/proxy.js';
export default function proxyMiddleware(webProxies) {
    return async (req, res, next) => {
        if (req.headers['proxied-by-ox-proxy'])
            return next();
        const path = parseurl(req)?.path;
        if (!path)
            return next();
        for (const webProxy of webProxies) {
            if (shouldHandle(webProxy.condition, path)) {
                try {
                    const { url: target, secure } = webProxy;
                    const newPath = createPath(path, webProxy);
                    // forwarded request falls back to http 1.x, so we must have a valid host header
                    const headers = createHeaders(req, webProxy);
                    logger.info(`Proxy ${path} to ${target.origin}${newPath}`);
                    logger.trace(`Set headers for ${path}: ${JSON.stringify(headers)}`);
                    await proxy.web(req, res, { url: target, secure, path: newPath, headers });
                    logger.debug(`Proxied ${path} to ${target.origin}${path}`);
                    return;
                }
                catch (err) {
                    const handle = finalhandler(req, res);
                    if (err instanceof HttpError) {
                        logger.warn(`Could not proxy ${path}`);
                        return handle(err);
                    }
                    logger.error(`Could not proxy ${path}: ${getStack(err)}`);
                    return handle();
                }
            }
        }
        next();
    };
}
